@@ -1,16 +1,16 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 
-#include <algorithm>
-
 #include "zeek-config.h"
-
 #include "XDR.h"
 
+#include <string.h>
+#include <algorithm>
+
 #include "events.bif.h"
 
 using namespace analyzer::rpc;
 
-uint32 analyzer::rpc::extract_XDR_uint32(const u_char*& buf, int& len)
+uint32_t analyzer::rpc::extract_XDR_uint32(const u_char*& buf, int& len)
 	{
 	if ( ! buf )
 		return 0;
@@ -21,8 +21,10 @@ uint32 analyzer::rpc::extract_XDR_uint32(const u_char*& buf, int& len)
 		return 0;
 		}
 
-	// Takes care of alignment and endianess differences. 
-	uint32 bits32 = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
+	// Takes care of alignment and endianess differences.
+	uint32_t buf_bits32;
+	memcpy(&buf_bits32, buf, 4);
+	uint32_t bits32 = ntohl(buf_bits32);
 
 	buf += 4;
 	len -= 4;
@@ -30,7 +32,7 @@ uint32 analyzer::rpc::extract_XDR_uint32(const u_char*& buf, int& len)
 	return bits32;
 	}
 
-uint64 analyzer::rpc::extract_XDR_uint64(const u_char*& buf, int& len)
+uint64_t analyzer::rpc::extract_XDR_uint64(const u_char*& buf, int& len)
 	{
 	if ( ! buf || len < 8 )
 		{
@@ -38,8 +40,8 @@ uint64 analyzer::rpc::extract_XDR_uint64(const u_char*& buf, int& len)
 		return 0;
 		}
 
-	uint64 uhi = extract_XDR_uint32(buf, len);
-	uint64 ulo = extract_XDR_uint32(buf, len);
+	uint64_t uhi = extract_XDR_uint32(buf, len);
+	uint64_t ulo = extract_XDR_uint32(buf, len);
 
 	return (uhi << 32) + ulo;
 	}
@@ -52,8 +54,8 @@ double analyzer::rpc::extract_XDR_time(const u_char*& buf, int& len)
 		return 0.0;
 		}
 
-	uint32 uhi = extract_XDR_uint32(buf, len);
-	uint32 ulo = extract_XDR_uint32(buf, len);
+	uint32_t uhi = extract_XDR_uint32(buf, len);
+	uint32_t ulo = extract_XDR_uint32(buf, len);
 
 	return double(uhi) + double(ulo) / 1e9;
 	}
@@ -101,9 +103,9 @@ const u_char* analyzer::rpc::extract_XDR_opaque_fixed(const u_char*& buf, int& l
 	}
 
 
-uint32 analyzer::rpc::skip_XDR_opaque_auth(const u_char*& buf, int& len)
+uint32_t analyzer::rpc::skip_XDR_opaque_auth(const u_char*& buf, int& len)
 	{
-	uint32 auth_flavor = extract_XDR_uint32(buf, len);
+	uint32_t auth_flavor = extract_XDR_uint32(buf, len);
 	if ( ! buf )
 		return 0;
 
